#include "massdata.h"

using namespace std;
extern double pi;
extern double U;

MassData::MassData(QWidget *parent) : QWidget(parent)
{
    QFile massFile("massdata/ame12.txt");
    QFile elementFile("massdata/element_names.txt");
    QString line,z,a,file,massExcess,massError,keyString,nString,nameString;
    int i,j,z_int,n_int;
    cout.precision(15);
    
    if (!massFile.open(QIODevice::ReadOnly | QIODevice::Text))
 QMessageBox::warning(this,tr("Error"), tr("There was a problem loading the mass data table."));
    
     if (!elementFile.open(QIODevice::ReadOnly | QIODevice::Text))
 QMessageBox::warning(this,tr("Error"), tr("There was a problem loading the element name table."));
    
    QTextStream data(&massFile);
    QTextStream elements(&elementFile);
    
    for (int i=0;i<=38;i++)  //read past headers
	data.readLine();
    
    j = 0;
                
    while (j <= 3313) //number of lines to read /wo headers
    {
 line = data.readLine();
    for (i=1; i<=3; i++)
    {
 a.append(line.at(i+15));
                z.append(line.at(i+10));
    }
 
    for (i=43;i<=51;i++)
                      massError.append(line.at(i));
  
    for (i=29; i<=40; i++)
                      massExcess.append(line.at(i));
 
                 z_int = z.toInt();
  n_int = a.toInt()-z_int;
                 a = "";
  z = "";
  
                 keyString.setNum(z_int);
                 keyString.append(nString.setNum(n_int));
   
  if(massExcess.contains("#"))
	massExcess.remove("#");
	
  if(massError.contains("#"))
	massError.remove("#");
  
  mapMass[keyString.toInt()] = massExcess.toDouble();
  mapError[keyString.toInt()] = massError.toDouble();
  massExcess = "";
  massError = "";
  
    j++;
    }
    
    for (j=0; j<=108; j++)
    {
 line = elements.readLine();
                nameString = line.remove(0,3);
 mapElements[j+1] = nameString;
 mapZ[nameString.simplified()] = j+1;
     }
}  

double MassData::readMass(int key)
{
    return mapMass[key];
}

double MassData::readMassError(int key)
{
    return mapError[key];
}
 
QString MassData::readElementName(int key)
{
    return mapElements[key];
}

int MassData::readElementZ(QString key)
{
    return mapZ[key];
}

QStringList MassData::parseMassString(QString massString)
{
    QStringList massList = massString.split(":");
    QStringList returnList;
    QString elementString,moleculeString,aString,multString,entryString,zString,nString,
                 keyString;
    int multiplier,a,z,n,length;
    double massexcess, masserror,totalmassexcess=0,totalmasserror=0;
    
    for (int i=0;i<=massList.count()-1;i++)
    {
	entryString=massList[i];
	length=entryString.length();
	aString="";
	elementString="";
	if (entryString.at(0)=='0' || entryString.at(0)=='1' || entryString.at(0)=='2' ||
	    entryString.at(0)=='3' || entryString.at(0)=='4' || entryString.at(0)=='5' ||
	    entryString.at(0)=='6' || entryString.at(0)=='7' || entryString.at(0)=='8' ||
	    entryString.at(0)=='9')
	{
	    multString=entryString.at(0);
	    multiplier=multString.toInt();
	}
	else 
	{
	    multiplier=1;
	    multString.setNum(1);
	    elementString.append(entryString.at(0));
	}
	
	for (int j=1;j<=length-1;j++)
	{
	    if (entryString.at(j)=='0' || entryString.at(j)=='1' || entryString.at(j)=='2' ||
	    entryString.at(j)=='3' || entryString.at(j)=='4' || entryString.at(j)=='5' ||
	    entryString.at(j)=='6' || entryString.at(j)=='7' || entryString.at(j)=='8' ||
	    entryString.at(j)=='9')
		aString.append(entryString.at(j));
	    else
		elementString.append(entryString.at(j));
	}
	                moleculeString.append(multString+":");
		moleculeString.append(elementString+":");
		moleculeString.append(aString+":");
		z=this->readElementZ(elementString);
		n=aString.toInt()-z;
		zString.setNum(z);
		nString.setNum(n);
		keyString=zString+nString;
		massexcess=this->readMass(keyString.toInt());
		masserror=this->readMassError(keyString.toInt());
		totalmassexcess+=multiplier*massexcess;
		totalmasserror+=multiplier*masserror*masserror;
        }
    
    QString totalMExString;
    totalMExString.setNum(totalmassexcess,'g',15);
    QString totalMErString;
    totalMErString.setNum(sqrt(totalmasserror),'g',15);
    returnList.append(moleculeString);
    returnList.append(totalMExString);
    returnList.append(totalMErString);
    
    return returnList;
}

QStringList MassData::parseMoleculeString(QString moleculeString)
{
    QStringList stringList;
    QRegExp moleculeRegExp = QRegExp::QRegExp("[A-Z]{1,1}[a-z]{1,2}");
    
    //int pos = moleculeRegExp.search(moleculeString,0);
    stringList = moleculeRegExp.capturedTexts();
    return stringList;
}

double MassData::wcInterpolation( double wc1, double wc2,
			       double massamu1, double massamu2, double 
			       refmassamu, double charge1, double charge2, double refcharge, 
			       double time1toM, double time1to2)
{
   /* double B1=massamu1*U*wc1/(2*pi*charge1*1.60217733e-19);
    double B2=massamu2*U*wc2/(2*pi*charge2*1.60217733e-19);
    
    double Binterp=(B2-B1)*(time1toM/time1to2)+B1;
    double refwcinterp=refcharge*1.60217733e-19*2*pi*Binterp/(refmassamu*U);*/
	
	double B1=massamu1*U*1000*wc1*2*pi/(charge1*299792458*299792458);
	double B2=massamu2*U*1000*wc2*2*pi/(charge2*299792458*299792458);
	
	double Binterp=(B2-B1)*(time1toM/time1to2)+B1;
	double refwcinterp=refcharge*Binterp*299792458*299792458/(2*pi*refmassamu*U*1000);
    
    return refwcinterp;
}

double MassData::wcInterpolation2(double wc1, double wc2, double timeMto1, double deltareftime)
{
    timeMto1 = timeMto1;
    deltareftime = deltareftime;
 
    double refwcinterp = (wc2-wc1)*(timeMto1/deltareftime)+wc1;
   
    return refwcinterp;
}

double MassData::interpUnc(double unc1, double time2, double unc2, double time_meas)
{
/*	double delta=(1/(unc1*unc1)+1/(unc2*unc2))*(time2*time2)/(unc2*unc2)
				-(time2/(unc2*unc2))*(time2/(unc2*unc2));
	double sigasq=(1/delta)*(time2*time2/(unc2*unc2));
	double sigbsq=(1/delta)*(1/(unc1*unc1)+1/(unc2*unc2));
	
	return sqrt(sigasq+sigbsq*time_meas*time_meas);*/
	
	double fac1 = 1-time_meas/time2;
	double fac2 = time_meas/time2;
	
	return sqrt(fac1*fac1*unc1*unc1+fac2*fac2*unc2*unc2);
}
