/*!
Upon initialization this class reads in the atomic mass evaluation file (here it's awm.txt located in the massdata directory), along with element_names.txt. If one changes the name of the file then a dialog should pop up and notify you that it can't find the file.  Perhaps a smart file loading will be implemented later.

Using this class one can read the mass and mass error of a given nucleus by passing an int key value of the form (ZN), so for hydrogen the key would be 10, or 12C would be 66.

The readElementName method takes in an integer corresponding to the Z of the nucleus and returns a QString containing the element's symbol name.  So passing 1 would return H, and 6 would return C.
*/

#ifndef MASSDATA_H
#define MASSDATA_H

#include <QString>
#include <QStringList>
#include <QTextStream>
#include <QFile>
#include <QWidget>
#include <QMessageBox>
#include <QRegExp>
#include <map>
#include <iostream>
#include <math.h>
#include "nr_recipes/nr.h"

class MassData : public QWidget
{
    Q_OBJECT
public:
    MassData(QWidget *parent = 0);
public:
    double readMass(int key);
    double readMassError(int key);
    QString readElementName(int key);
    int readElementZ(QString key);
    QStringList parseMassString(QString massString);
    QStringList parseMoleculeString(QString moleculeString);
    double wcInterpolation( double wc1, double wc2, double massamu1, 
		          double massamu2 ,double refmassamu, double charge1, double 
				  charge2, double refcharge, double time1toM, double time1to2);
    double wcInterpolation2(double wc1, double wc2, double timeMto1, double deltareftime);
	double interpUnc(double unc1, double time2, double unc2, double time_meas);
private:
    map<int, double> mapMass;
    map<int, double> mapError;
    map<int, QString> mapElements;
    map<QString, int> mapZ;
};

#endif
